@@ -15,37 +15,23 @@
 #         `=. `=./
 #            `"`
 from optparse import OptionParser
+
 import re
 import sys
 
+from peewee import *
+
 try:
     from MySQLdb.constants import FIELD_TYPE
-    MYSQL_MAP = {
-        FIELD_TYPE.BLOB: 'TextField',
-        FIELD_TYPE.CHAR: 'CharField',
-        FIELD_TYPE.DECIMAL: 'DecimalField',
-        FIELD_TYPE.NEWDECIMAL: 'DecimalField',
-        FIELD_TYPE.DATE: 'DateField',
-        FIELD_TYPE.DATETIME: 'DateTimeField',
-        FIELD_TYPE.DOUBLE: 'FloatField',
-        FIELD_TYPE.FLOAT: 'FloatField',
-        FIELD_TYPE.INT24: 'IntegerField',
-        FIELD_TYPE.LONG: 'IntegerField',
-        FIELD_TYPE.LONGLONG: 'BigIntegerField',
-        FIELD_TYPE.SHORT: 'IntegerField',
-        FIELD_TYPE.STRING: 'CharField',
-        FIELD_TYPE.TIME: 'TimeField',
-        FIELD_TYPE.TIMESTAMP: 'DateTimeField',
-        FIELD_TYPE.TINY: 'IntegerField',
-        FIELD_TYPE.TINY_BLOB: 'TextField',
-        FIELD_TYPE.MEDIUM_BLOB: 'TextField',
-        FIELD_TYPE.LONG_BLOB: 'TextField',
-        FIELD_TYPE.VAR_STRING: 'CharField',
-    }
 except ImportError:
-    MYSQL_MAP = {}
+    try:
+        from pymysql.constants import FIELD_TYPE
+    except ImportError:
+        FIELD_TYPE = None
 
-from peewee import *
+
+class UnknownFieldType(object):
+    pass
 
 
 class DB(object):
@@ -59,8 +45,8 @@ class DB(object):
         get_columns('some_table')
 
         {
-            'name': 'CharField',
-            'age': 'IntegerField',
+            'name': CharField,
+            'age': IntegerField,
         }
         """
         raise NotImplementedError
@@ -93,22 +79,22 @@ class DB(object):
 class PgDB(DB):
     # thanks, django
     reverse_mapping = {
-        16: 'BooleanField',
-        20: 'IntegerField',
-        21: 'IntegerField',
-        23: 'IntegerField',
-        25: 'TextField',
-        700: 'FloatField',
-        701: 'FloatField',
-        1042: 'CharField', # blank-padded CHAR
-        1043: 'CharField',
-        1082: 'DateField',
-        1114: 'DateTimeField',
-        1184: 'DateTimeField',
-        1083: 'TimeField',
-        1266: 'TimeField',
-        1700: 'DecimalField',
-        2950: 'TextField', # UUID
+        16: BooleanField,
+        20: IntegerField,
+        21: IntegerField,
+        23: IntegerField,
+        25: TextField,
+        700: FloatField,
+        701: FloatField,
+        1042: CharField, # blank-padded CHAR
+        1043: CharField,
+        1082: DateField,
+        1114: DateTimeField,
+        1184: DateTimeField,
+        1083: TimeField,
+        1266: TimeField,
+        1700: DecimalField,
+        2950: TextField, # UUID
     }
 
     def get_conn_class(self):
@@ -116,7 +102,7 @@ class PgDB(DB):
 
     def get_columns(self, table):
         curs = self.conn.execute_sql('select * from "%s" limit 1' % table)
-        return dict((c.name, self.reverse_mapping.get(c.type_code, 'UnknownFieldType')) for c in curs.description)
+        return dict((c.name, self.reverse_mapping.get(c.type_code, UnknownFieldType)) for c in curs.description)
 
     def get_foreign_keys(self, table, schema='public'):
         framing = '''
@@ -142,14 +128,38 @@ class PgDB(DB):
 
 class MySQLDB(DB):
     # thanks, django
-    reverse_mapping = MYSQL_MAP
+    if FIELD_TYPE is None:
+        reverse_mapping = {}
+    else:
+        reverse_mapping = {
+            FIELD_TYPE.BLOB: TextField,
+            FIELD_TYPE.CHAR: CharField,
+            FIELD_TYPE.DECIMAL: DecimalField,
+            FIELD_TYPE.NEWDECIMAL: DecimalField,
+            FIELD_TYPE.DATE: DateField,
+            FIELD_TYPE.DATETIME: DateTimeField,
+            FIELD_TYPE.DOUBLE: FloatField,
+            FIELD_TYPE.FLOAT: FloatField,
+            FIELD_TYPE.INT24: IntegerField,
+            FIELD_TYPE.LONG: IntegerField,
+            FIELD_TYPE.LONGLONG: BigIntegerField,
+            FIELD_TYPE.SHORT: IntegerField,
+            FIELD_TYPE.STRING: CharField,
+            FIELD_TYPE.TIME: TimeField,
+            FIELD_TYPE.TIMESTAMP: DateTimeField,
+            FIELD_TYPE.TINY: IntegerField,
+            FIELD_TYPE.TINY_BLOB: TextField,
+            FIELD_TYPE.MEDIUM_BLOB: TextField,
+            FIELD_TYPE.LONG_BLOB: TextField,
+            FIELD_TYPE.VAR_STRING: CharField,
+        }
 
     def get_conn_class(self):
         return MySQLDatabase
 
     def get_columns(self, table):
         curs = self.conn.execute_sql('select * from `%s` limit 1' % table)
-        return dict((r[0], self.reverse_mapping.get(r[1], 'UnknownFieldType')) for r in curs.description)
+        return dict((r[0], self.reverse_mapping.get(r[1], UnknownFieldType)) for r in curs.description)
 
     def get_foreign_keys(self, table):
         framing = '''
@@ -166,22 +176,22 @@ class MySQLDB(DB):
 class SqDB(DB):
     # thanks, django
     reverse_mapping = {
-        'bool': 'BooleanField',
-        'boolean': 'BooleanField',
-        'smallint': 'IntegerField',
-        'smallint unsigned': 'IntegerField',
-        'smallinteger': 'IntegerField',
-        'int': 'IntegerField',
-        'integer': 'IntegerField',
-        'bigint': 'BigIntegerField',
-        'integer unsigned': 'IntegerField',
-        'decimal': 'DecimalField',
-        'real': 'FloatField',
-        'text': 'TextField',
-        'char': 'CharField',
-        'date': 'DateField',
-        'datetime': 'DateTimeField',
-        'time': 'TimeField',
+        'bool': BooleanField,
+        'boolean': BooleanField,
+        'smallint': IntegerField,
+        'smallint unsigned': IntegerField,
+        'smallinteger': IntegerField,
+        'int': IntegerField,
+        'integer': IntegerField,
+        'bigint': BigIntegerField,
+        'integer unsigned': IntegerField,
+        'decimal': DecimalField,
+        'real': FloatField,
+        'text': TextField,
+        'char': CharField,
+        'date': DateField,
+        'datetime': DateTimeField,
+        'time': TimeField,
     }
 
     def get_conn_class(self):
@@ -192,9 +202,9 @@ class SqDB(DB):
         if col in self.reverse_mapping:
             return self.reverse_mapping[col]
         elif re.search(r'^\s*(?:var)?char\s*\(\s*(\d+)\s*\)\s*$', col):
-            return 'CharField'
+            return CharField
         else:
-            return 'UnknownFieldType'
+            return UnknownFieldType
 
     def get_columns(self, table):
         curs = self.conn.execute_sql('pragma table_info(%s)' % table)
@@ -202,7 +212,7 @@ class SqDB(DB):
         for (_, name, col, not_null, _, is_pk) in curs.fetchall():
             # cid, name, type, notnull, dflt_value, pk
             if is_pk:
-                col_type = 'PrimaryKeyField'
+                col_type = PrimaryKeyField
             else:
                 col_type = self.map_col(col)
             col_dict[name] = col_type
@@ -250,24 +260,22 @@ engine_mapping = {
     'mysql': MySQLDB,
 }
 
-def get_db(engine):
+def get_conn(engine, database, **connect):
     if engine not in engine_mapping:
         err('Unsupported engine: "%s"' % engine)
         sys.exit(1)
 
-    db_class = engine_mapping[engine]
-    return db_class()
-
-def introspect(engine, database, **connect):
-    db = get_db(engine)
+    db = engine_mapping[engine]()
     schema = connect.pop('schema', None)
     db.connect(database, **connect)
 
     if schema:
         db.conn.set_search_path(*schema.split(','))
+    return db
 
+def introspect(db, schema=None):
     tables = db.get_tables()
-
+    
     models = {}
     table_to_model = {}
     table_fks = {}
@@ -277,7 +285,7 @@ def introspect(engine, database, **connect):
         models[table] = db.get_columns(table)
         table_to_model[table] = tn(table)
         if schema:
-            table_fks[table] = db.get_foreign_keys(table,schema)
+            table_fks[table] = db.get_foreign_keys(table, schema)
         else:
             table_fks[table] = db.get_foreign_keys(table)
 
@@ -287,36 +295,53 @@ def introspect(engine, database, **connect):
     for table in tables:
         col_meta[table] = {}
         for column, rel_table, rel_pk in table_fks[table]:
-            models[table][column] = 'ForeignKeyField'
-            models[rel_table][rel_pk] = 'PrimaryKeyField'
-            col_meta[table][column] = {'rel_model': table_to_model[rel_table]}
+            models[table][column] = ForeignKeyField
+            models[rel_table][rel_pk] = PrimaryKeyField
+            if rel_table == table:
+                ttm = "'self'"
+            else:
+                ttm = table_to_model[rel_table]
+            col_meta[table][column] = {'rel_model': ttm}
 
         for column in models[table]:
             col_meta[table].setdefault(column, {})
             if column != cn(column):
                 col_meta[table][column]['db_column'] = "'%s'" % column
+    
+    return models, table_to_model, table_fks, col_meta
 
+def print_models(engine, database, **connect):
+    schema = connect.get('schema')
+    db = get_conn(engine, database, **connect)
+    
+    models, table_to_model, table_fks, col_meta = introspect(db, schema)
+    
     # write generated code to standard out
     print frame % (db.get_conn_class().__name__, database, repr(connect))
 
     # print the models
-    def print_model(model, seen):
+    def print_model(model, seen, accum=None):
+        accum = accum or []
+
         for _, rel_table, _ in table_fks[model]:
-            if rel_table not in seen:
+            if rel_table in accum and model not in accum:
+                print '# POSSIBLE REFERENCE CYCLE: %s' % table_to_model[rel_table]
+
+            if rel_table not in seen and rel_table not in accum:
                 seen.add(rel_table)
-                print_model(rel_table, seen)
+                print_model(rel_table, seen, accum + [model])
 
         ttm = table_to_model[model]
         print 'class %s(BaseModel):' % ttm
         cols = models[model]
         for column, field_class in ds(cols):
-            if column == 'id' and field_class in ('IntegerField', 'PrimaryKeyField'):
+            if column == 'id' and field_class in (IntegerField, PrimaryKeyField):
                 continue
 
             field_params = ', '.join([
                 '%s=%s' % (k, v) for k, v in col_meta[model][column].items()
             ])
-            print '    %s = %s(%s)' % (cn(column), field_class, field_params)
+            print '    %s = %s(%s)' % (cn(column), field_class.__name__, field_params)
         print
 
         print '    class Meta:'
@@ -362,4 +387,4 @@ if __name__ == '__main__':
     if options.engine == 'mysql' and 'password' in connect:
         connect['passwd'] = connect.pop('password', None)
 
-    introspect(options.engine, database, **connect)
+    print_models(options.engine, database, **connect)
